==> techutils-0.6.0%2B1.21.10-sources.jar: dev/kikugie/techutils/feature/worldedit/WorldEditNetworkHandler.java <==
package dev.kikugie.techutils.feature.worldedit;

import dev.kikugie.techutils.TechUtilsMod;
import net.fabricmc.fabric.api.client.networking.v1.ClientPlayNetworking;
import net.minecraft.class_2338;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_8710;
import org.enginehub.worldeditcui.protocol.CUIPacket;

import java.util.Optional;

/**
 * Opens a WorldEdit CUI channel and passes packets to {@link WorldEditSync} instance.
 * <p>
 * If channel has already been registered by another mod, it leeches packets from it instead.
 */
public class WorldEditNetworkHandler {
	public static final class_2960 CHANNEL = class_2960.method_60655("worldedit", "cui");
	private static final int PROTOCOL = 4;
	private static WorldEditNetworkHandler instance;
	public final WorldEditStorage storage;
	private boolean worldEditConnected = false;
	private boolean yoinkPackets = false;

	private WorldEditNetworkHandler() {
		this.storage = WorldEditStorage.init();
		var receivers = ClientPlayNetworking.getGlobalReceivers();
		if (receivers.contains(CHANNEL)) {
			this.yoinkPackets = true;
			handshake();
			return;
		}
		ClientPlayNetworking.registerReceiver(CUIPacket.TYPE, (payload, context) -> this.onPacket(payload));
		handshake();
	}

	public static WorldEditNetworkHandler initHandler() {
		if (class_310.method_1551().field_1687 == null) {
			throw new RuntimeException("Registering WorldEdit handler in non-world context!");
		}
		instance = new WorldEditNetworkHandler();
		return instance;
	}

	public static Optional<WorldEditNetworkHandler> getInstance() {
		return Optional.ofNullable(instance);
	}

	public void onYoinkedPacket(class_8710 payload) {
		if (!this.yoinkPackets) return;
		onPacket((CUIPacket) payload);
	}

	private void onPacket(CUIPacket packet) {
		if (!this.worldEditConnected) {
			this.worldEditConnected = true;
			TechUtilsMod.LOGGER.info("WorldEdit connected");
			WorldEditSync.getInstance().ifPresent(WorldEditSync::onWorldEditConnected);
		}
		if (packet.args().isEmpty()) {
			TechUtilsMod.LOGGER.warn("WorldEditNetworkHandler#onPacket(): Received CUI packet of length zero");
			return;
		}
		handlePacket(packet.eventType(), packet.args().toArray(String[]::new));
	}

	private void handlePacket(String type, String[] args) {
		if (type.equals("s")) {
			this.storage.setCuboid(args[0].equals("cuboid"));
			return;
		}
		if (!type.equals("p") || !this.storage.isCuboid())
			return;
		try {
			int p = Integer.parseInt(args[0]);
			class_2338 pos = new class_2338(Integer.parseInt(args[1]), Integer.parseInt(args[2]), Integer.parseInt(args[3]));
			this.storage.setPos(p == 0, pos);
		} catch (NumberFormatException e) {
			TechUtilsMod.LOGGER.warn("WorldEditCUINetworkHandler#handlePacket(): Failed int parsing of position", e);
		}
	}

	private void handshake() {
		ClientPlayNetworking.send(new CUIPacket("v", String.valueOf(PROTOCOL)));
	}
}

==> techutils-0.6.1%2B1.21.1-sources.jar: dev/kikugie/techutils/feature/worldedit/WorldEditNetworkHandler.java <==
package dev.kikugie.techutils.feature.worldedit;

import com.sk89q.worldedit.fabric.net.handler.WECUIPacketHandler;
import dev.kikugie.techutils.TechUtilsMod;
import net.fabricmc.fabric.api.client.networking.v1.ClientPlayNetworking;
import net.minecraft.class_2338;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_8710;
import java.util.Optional;

/**
 * Opens a WorldEdit CUI channel and passes packets to {@link WorldEditSync} instance.
 * <p>
 * If channel has already been registered by another mod, it leeches packets from it instead.
 */
public class WorldEditNetworkHandler {
	public static final class_2960 CHANNEL = class_2960.method_60655("worldedit", "cui");
	private static final int PROTOCOL = 4;
	private static WorldEditNetworkHandler instance;
	public final WorldEditStorage storage;
	private boolean worldEditConnected = false;
	private boolean yoinkPackets = false;

	private WorldEditNetworkHandler() {
		this.storage = WorldEditStorage.init();
		var receivers = ClientPlayNetworking.getGlobalReceivers();
		if (receivers.contains(CHANNEL)) {
			this.yoinkPackets = true;
			return;
		}
		ClientPlayNetworking.registerReceiver(WECUIPacketHandler.CuiPacket.TYPE, (payload, context) -> this.onPacket(payload.text()));
		handshake();
	}

	public static WorldEditNetworkHandler initHandler() {
		if (class_310.method_1551().field_1687 == null) {
			throw new RuntimeException("Registering WorldEdit handler in non-world context!");
		}
		instance = new WorldEditNetworkHandler();
		return instance;
	}

	public static Optional<WorldEditNetworkHandler> getInstance() {
		return Optional.ofNullable(instance);
	}

	public void onYoinkedPacket(class_8710 payload) {
		if (!this.yoinkPackets) return;
		onPacket(((WECUIPacketHandler.CuiPacket) payload).text());
	}

	private void onPacket(String message) {
		if (!this.worldEditConnected) {
			this.worldEditConnected = true;
			TechUtilsMod.LOGGER.info("WorldEdit connected");
			WorldEditSync.getInstance().ifPresent(WorldEditSync::onWorldEditConnected);
		}
		if (message.isEmpty()) {
			TechUtilsMod.LOGGER.warn("WorldEditNetworkHandler#onPacket(): Received CUI packet of length zero");
			return;
		}
		String[] split = message.split("\\|", -1);
		boolean multi = split[0].startsWith("+");
		String type = split[0].substring(multi ? 1 : 0);
		String[] args = message.substring(type.length() + (multi ? 2 : 1)).split("\\|", -1);
		TechUtilsMod.LOGGER.info(message);
		handlePacket(type, args);
	}

	private void handlePacket(String type, String[] args) {
		if (type.equals("s")) {
			this.storage.setCuboid(args[0].equals("cuboid"));
			return;
		}
		if (!type.equals("p") || !this.storage.isCuboid())
			return;
		try {
			int p = Integer.parseInt(args[0]);
			class_2338 pos = new class_2338(Integer.parseInt(args[1]), Integer.parseInt(args[2]), Integer.parseInt(args[3]));
			this.storage.setPos(p == 0, pos);
		} catch (NumberFormatException e) {
			TechUtilsMod.LOGGER.warn("WorldEditCUINetworkHandler#handlePacket(): Failed int parsing of position");
			e.printStackTrace();
		}
	}

	private void handshake() {
		String message = "v|" + PROTOCOL;
		ClientPlayNetworking.send(new WECUIPacketHandler.CuiPacket(message));
	}
}
